# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil

from ksearch.crawler import queue

class QueueTestCase(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self.queue = queue.BTreeQueue(self.tmp_dir)

    def tearDown(self):
        del self.queue
        shutil.rmtree(self.tmp_dir)

    def test_prepend(self):
        test = 'test text'
        test2 = 'test text2'
        test3 = 'test text3'
        self.queue.append(test)
        self.queue.prepend(test2)
        self.queue.prepend(test3)
        self.queue.prepend(test)
        self.queue.prepend(test2)
        ret = self.queue.consume()
        self.assertEquals(ret, test2)
        ret = self.queue.consume()
        self.assertEquals(ret, test)
        ret = self.queue.consume()
        self.assertEquals(ret, test3)
        ret = self.queue.consume()
        self.assertEquals(ret, test2)

    def test_append(self):
        test = 'test text'
        test2 = 'test text2'
        test3 = 'test text3'
        self.queue.append(test)
        self.queue.append(test2)
        self.queue.append(test3)
        ret = self.queue.consume()
        self.assertEquals(ret, test)
        ret = self.queue.consume()
        self.assertEquals(ret, test2)
        ret = self.queue.consume()
        self.assertEquals(ret, test3)

    def test_consume(self):
        test = 'test text'
        test2 = 'test text2'
        test3 = 'test text3'
        self.queue.append(test)
        self.queue.append(test2)
        self.queue.append(test3)
        ret = self.queue.consume()
        self.assertEquals(ret, test)
        ret = self.queue.consume()
        self.assertEquals(ret, test2)
        ret = self.queue.consume()
        self.assertEquals(ret, test3)
        ret = self.queue.consume()
        self.assertEquals(ret, None)

    def test_empty(self):
        ret = self.queue.empty()
        self.assertEquals(ret, True)
        self.queue.append('test text')
        ret = self.queue.empty()
        self.assertEquals(ret, False)

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(QueueTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
