# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil

from ksearch.crawler import storage

class StorageTestCase(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self.storage = storage.Storage(self.tmp_dir)

    def tearDown(self):
        del self.storage
        shutil.rmtree(self.tmp_dir)

    def test_pagerank(self):
        self.storage.set_pagerank([(0, 1.2), (1, 1.0)])
        rank = self.storage.get_pagerank(0)
        self.assertEquals(rank, '1.2')
        rank = self.storage.get_pagerank(1)
        self.assertEquals(rank, '1.0')

    def test_reopen(self):
        self.storage.close()
        self.storage = storage.Storage(self.tmp_dir)
        url = 'test text'
        data = 'test text2'
        i = self.storage.add(url, data, 'text/plain')

    def test_get_last_id(self):
        url = 'test text'
        data = 'test text2'
        i = self.storage.add(url, data, 'text/plain')
        ret = self.storage.get_last_id()
        self.assertEquals(ret, 1)

    def test_add(self):
        url = 'test text'
        data = 'test text2'
        i = self.storage.add(url, data, 'text/plain')
        ret = self.storage.get_url(i)
        self.assertEquals(ret, url)
        ret = self.storage.get_data(i)
        self.assertEquals(ret, data)

    def test_modify(self):
        url = data = 'test text'
        id = self.storage.add(url, 'test text2', 'text/plain')
        self.storage.modify(id, data)
        ret = self.storage.get_data(id)
        self.assertEquals(ret, data)

    def test_delete(self):
        url = data2 ='test text'
        data = url2 ='test text2'
        self.storage.add(url, data, 'text/plain')
        i = self.storage.add(url2, data2, 'text/plain')
        ret = self.storage.get_url(i)
        self.assertEquals(ret, url2)
        self.storage.delete(i)
        ret = self.storage.get_url(i)
        self.assertEquals(ret, None)

    def test_get_url(self):
        ret = self.storage.get_url(1234)
        self.assertEquals(ret, None)

    def test_get_type(self):
        ret = self.storage.get_type(1234)
        self.assertEquals(ret, None)
    
    def test_get_data(self):
        ret = self.storage.get_data(1234)
        self.assertEquals(ret, None)

    def test_get_id(self):
        url = 'test text'
        ret = self.storage.get_id("1234")
        self.assertEquals(ret, None)
        id = self.storage.add(url, 'test text2', 'text/plain')
        ret = self.storage.get_id(url)
        self.assertEquals(ret, id)

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(StorageTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
