#-*- coding: utf-8 -*-
import sys
import mimetypes
import re
import stat as fstat
import posixpath, os, os.path
import cgi
import time
from email.Utils import parsedate_tz, mktime_tz
import logging
import urllib

from ksearch.util import get_object, query2dict, http_date
from ksearch.frontend.framework import *
from ksearch.frontend.template import loader
from ksearch.frontend.template.context import Context

REALM_NAMES = {
    'web': u'카이스트 웹',
    'ara': u'아라',
    'portal': u'포탈',
}

class Statistics(object):
    """통계 페이지를 위한 Mock-up 클래스"""

    def get_pages_to_crawl(self):
        return 1200

    def get_pages_crawled(self):
        return 1000

    def get_pages_analyzed(self):
        return 700

    def get_pages_to_analyze(self):
        return 800

    def get_recent_user_queries(self):
        return [u'ara', u'kaist', u'sparcs', u'웹아라', u'스팍스',
                u'택배', u'야식']

    def get_pages_crawled_by_time(self):
        return {1123570:120, 1123575:140, 1123580:165, 1123585:175,
                1123590:190, 1123595:210, 1123600:230}

class Search(object):
    """진짜 검색 객체를 대신하는 테스트용 Mock-up 클래스"""

    def search(self, q, from_=0, to_=100, source='', sort=''):
        """가짜 검색 결과를 내보내는 메소드."""
        if q == '': return []
        mock = {
        'hits': [
         {'uri':'http://example.com',
          'title':'Example Site',
          'content':'This is the example site of the century...'},
         {'uri':'http://apple.com',
          'title':'Apple Computer Inc.',
          'content':'Apple Computer Incorporated. Welcome...'},
         {'uri':'http://sparcs.kaist.ac.kr',
          'title':u'KAIST SPARCS 동아리',
          'content':u'스팍스 동아리에 오신것을 환영합니다...'},
         {'uri':'http://sparcs.kaist.ac.kr',
          'title':u'KAIST SPARCS 동아리',
          'content':u'스팍스 동아리에 오신것을 환영합니다...'},
         {'uri':'http://example.com',
          'title':'Example Site',
          'content':'This is the example site of the century...'},
         {'uri':'http://apple.com',
          'title':'Apple Computer Inc.',
          'content':'Apple Computer Incorporated. Welcome...'},
         {'uri':'http://example.com',
          'title':'Example Site',
          'content':'This is the example site of the century...'},
         {'uri':'http://apple.com',
          'title':'Apple Computer Inc.',
          'content':'Apple Computer Incorporated. Welcome...'},
         {'uri':'http://sparcs.kaist.ac.kr',
          'title':u'KAIST SPARCS 동아리',
          'content':u'스팍스 동아리에 오신것을 환영합니다...'},
         ],
         'search_whole_time': 0.1,
         'results': 9,
        }
        return mock

def _google_plot(keys, values):
    from pygooglechart import GroupedHorizontalBarChart, Axis

    if max(values) == 0:  # XXX(serialx): HACK FOR max(values) == 0
        chart = GroupedHorizontalBarChart(600, len(keys)*25 + 25, auto_scale=False)
    else:
        chart = GroupedHorizontalBarChart(600, len(keys)*25 + 25, auto_scale=True)
    chart.set_bar_width(20)
    chart.set_bar_spacing(5)
    chart.set_group_spacing(4)
    chart.set_colours(['66ccff', '66ffcc', 'ccff66', 'ffcc66', 'ff66cc', 'cc66ff'])
    for value in values:
        chart.add_data([value])
    chart.set_legend(keys)
    index = chart.set_axis_range(Axis.BOTTOM, 0, max(values))
    chart.set_axis_style(index, '202020', font_size=10, alignment=0)
    return chart.get_url()

def _get_statistics_about_time_data(database, statistic_groups, key, name, range=3600):
    start = int(time.time() * 1000 - range*1000)
    end = int(time.time() * 1000)
    data = database.get_index_range('statistics', key, start, end)
    index_times = [float(x[1]) for x in data]
    keys = [
        'Min %s Time' % name,
        'Avg %s Time' % name,
        'Median %s Time' % name,
        'Max %s Time' % name,
    ]
    values = [
        min(index_times),
        sum(index_times) / len(index_times),
        index_times[len(index_times)/2],
        max(index_times),
    ]
    img_src = _google_plot(keys, values)
    statistic_groups.append({'title': '%s Time Statistics (Last %d seconds)' % (name, range),
                             'statistics': zip(keys, values),
                             'img_src': img_src})
def now(request):
    database = get_object('database', object_namespace)
    statistic_groups = []

    keys = [
        'Index Queue Size',
        'Indexed Pages',
        'Indexed Pages In Memory',
    ]
    values = [
        int(database.get('statistics', 'total_idx_queue_size') or '0'),
        int(database.get('statistics', 'total_indexed_page') or '0'),
        int(database.get('statistics', 'total_indexed_page_in_memory') or '0'),
    ]
    img_src = _google_plot(keys, values)
    statistic_groups.append({'title': 'Indexing Status',
                             'statistics': zip(keys, values),
                             'img_src': img_src})

    keys = [
        'URL Queue Size',
        'Stored Pages',
    ]
    values = [
        int(database.get('statistics', 'total_queue_size') or '0'),
        int(database.get('statistics', 'total_stored_page') or '0'),
    ]
    img_src = _google_plot(keys, values)
    statistic_groups.append({'title': 'Crawling Status',
                             'statistics': zip(keys, values),
                             'img_src': img_src})

    _get_statistics_about_time_data(database, statistic_groups, 'search_time', 'Search')
    _get_statistics_about_time_data(database, statistic_groups, 'search_whole_time', 'Search Whole')
    _get_statistics_about_time_data(database, statistic_groups, 'index_time', 'Index')
    _get_statistics_about_time_data(database, statistic_groups, 'scheduling_time', 'Scheduling')
    _get_statistics_about_time_data(database, statistic_groups, 'crawl_time', 'Crawl')
    _get_statistics_about_time_data(database, statistic_groups, 'crawl_outlinks_time', 'Crawl Outlinks')
    _get_statistics_about_time_data(database, statistic_groups, 'total_idx_queue_size', 'Index Queue Size')
    _get_statistics_about_time_data(database, statistic_groups, 'total_indexed_page_in_memory', 'Memory Index Count')

    t = loader.get_template('now.html')
    c = Context({'page_name': 'now', 'statistic_groups': statistic_groups})
    return Response(body=t.render(c))


def stat(request):
    GET = query2dict(request['QUERY_STRING'])
    if 'range' not in GET:
        GET['range'] = 60*60

    output_template = Template(filepath='templates/statistics.html')
    return Response(body=output_template.substitute(
        {
            'range': GET['range'],
        }
        ))

def main(request):
    d = query2dict(request['QUERY_STRING'])
    d.setdefault('realm', 'web')
    """검색의 첫 페이지."""
    t = loader.get_template('main.html')
    c = Context({'page_name': 'main', 'realm': d['realm'], 'realms': REALM_NAMES})
    return Response(body=t.render(c))

def vote(uri):
    database = get_object('database', object_namespace)
    database.increment("vote", uri)

def go(request):
    """서버에 client의 방문 페이지 log를 남기기 위한 보조 메소드.
       /go?http://~ -> http://~ 로 연결됨
    """
    # output_template = Template(filepath=request['QUERY_STRING'])
    url = request['QUERY_STRING']
    logging.info('User URL Request: %s', url)
    return ResponseRedirect(redirect_url=url)

def pages(request):
    database = get_object('database', object_namespace)
    items = database.items('stored_pages')

    rows = []
    for host, pages in items:
        link = '/plot?namespace=stored_pages&key=' + host
        row = Template(u'''\
<tr>
<td>$host</td>
<td>$pages</td>
<td><a href="$link">Graph</a></td>
</tr>
''').substitute(locals())
        rows.append(row)
    rows = ''.join(rows)

    output = Template(filepath='templates/pages.html')
    return Response(body=output.substitute(rows=rows))

def plot(request):
    from PIL import Image as PILImage
    from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
    from matplotlib.figure import Figure
    from matplotlib.dates import date2num, HourLocator, DateFormatter
    from StringIO import StringIO

    dic = query2dict(request['QUERY_STRING'])
    try:
        namespace = dic['namespace']
        key = dic['key']
    except KeyError:
        return Response(body="Error: Invalid Argument")
    range = 60*60
    if 'range' in dic:
        range = int(dic['range'])

    database = get_object('database', object_namespace)
    start = int(time.time() * 1000 - range*1000)
    end = int(time.time() * 1000)
    plot_data = database.get_index_range(namespace, key, start, end)
    from datetime import datetime

    try:
        data_values = [float(x[1]) for x in plot_data]
        time_values = [date2num(datetime.fromtimestamp(float(x[0].rsplit(":",1)[1])/1000)) for x in plot_data]
    except TypeError:
        data_values = [0]
        time_values = [date2num(datetime.fromtimestamp(time.time()))]
    except ValueError:
        data_values = [0]
        time_values = [date2num(datetime.fromtimestamp(time.time()))]
    except IndexError:
        data_values = [0]
        time_values = [date2num(datetime.fromtimestamp(time.time()))]

    fig = Figure(facecolor="white", dpi=50)
    fig.set_figwidth(12)
    fig.set_figheight(3)

    canvas = FigureCanvas(fig)
    ax = fig.add_subplot(111)
    ax.grid(True)
    ax.plot_date(time_values, data_values, 'r-', xdate=True, ydate=False)
    ax.set_xlim(date2num(datetime.fromtimestamp(float(start)/1000)),
                date2num(datetime.fromtimestamp(float(end)/1000)))
    hour_fmt = DateFormatter("%H:%M")
    ax.xaxis.set_major_formatter(hour_fmt)
    ax.set_title(namespace + ":" + key)
    fig.autofmt_xdate()
    canvas.draw()

    size = canvas.get_renderer().get_canvas_width_height()
    buf = canvas.tostring_rgb()
    im = PILImage.fromstring('RGB', size, buf, 'raw', 'RGB', 0, 1)
    imdata = StringIO()

    im.save(imdata, format='PNG')
    return Response(body=imdata.getvalue(), headers={'Content-type': 'image/png'})


def handle(request):
    """검색 결과를 보여주는 화면에 관한 메소드"""

    d = query2dict(request['QUERY_STRING'])  # address parsing
    d.setdefault('q', '')
    d.setdefault('realm', 'web')
    d.setdefault('sort', 'score')
    word = d['q']
    realm = d['realm']
    sort = d['sort']
    if not sort in ('date', 'score'): sort='score'
    realm_name = REALM_NAMES[realm]

    # FORDEBUG: search = Search()
    result_buf = u''
    found_anything = False
    if word:
        search = get_object('searcher_service', object_namespace)
        #search = Search() # to use a mock-up class

        try:
            page = int( d['p'] )
        except KeyError:
            page = 1

        p_page = (page-1) / 10 + 1 # p_page : page partition

        time_before = time.time()
        from_ = (page - 1) * 10
        to_ = (page) * 10
        try:
            result = search.search(word, from_=from_, to_=to_, source=realm, sort=sort)
        except Pyro.errors.ConnectionClosedError, e:
            logging.warn('Connection lost. REBINDING... '
                         '(restart the server now)')
            search.adapter.rebindURI(tries=1, wait=1)
            result = search.search(word, from_=from_, to_=to_, source=realm, sort=sort)
        from_ = from_+1
        time_after = time.time()

        for x in result["hits"]:
            for k, v in x.items():  # k : uri / title / content
                v = unicode(v)
                x[k] = v
        cnt = result["results"]
        duration2 = "%1.4f" % result['search_whole_time']

        if cnt > 0:
            pages = u''
            page_st = 10 * (p_page-1) + 1
            page_en = 10 * p_page
            page_last = (cnt-1)/10 + 1

            if page_en > page_last:
                page_en = page_last

            s2=Template(u''' <a href="/search?realm=$realm&amp;q=$word&amp;p=$pg&amp;sort=$sort">$prt</a> ''')
            word_encoded = urllib.quote(word.encode('utf8'))
            if page_st != 1:
                pages += s2.substitute(word=word_encoded, pg=1, prt='&laquo;', realm=realm, sort=sort)
                pages += s2.substitute(word=word_encoded, pg=page_st-1, prt='&lt;', realm=realm, sort=sort)

            for i in range(page_st, page_en+1 ):
                if i == page:
                    tmp_i = '<strong>%d</strong>' % i    # 현재 페이지는 굵게 
                else:
                    tmp_i = str(i)
                pages += s2.substitute(word=word_encoded, pg=i, prt=tmp_i, realm=realm, sort=sort)

            if page_en != page_last:
                pages += s2.substitute(word=word_encoded, pg=page_en+1, prt='&gt;', realm=realm, sort=sort)
                pages += s2.substitute(word=word_encoded, pg=page_last, prt='&raquo;', realm=realm, sort=sort)

            if page == page_last:
                to_ = cnt
            rangec = '%d - %d' % (from_, to_)
            results = result['hits']
            found_anything = True
    else:
        duration2 = '0'

    if not found_anything:
        cnt = 0
        pages = u'(없음)'
        rangec = '0 - 0'
        results = None
        result = None

    t = loader.get_template('search.html')
    c = Context({
        'page_name': 'results',
        'results': results,
        'word': word,
        'pages': pages,
        'range': rangec,
        'count': cnt,
        'duration': duration2,
        'raw_result': result,
        'realm': realm,
        'realm_name': realm_name,
        'realms': REALM_NAMES,
        'sort': sort,
    })
    return Response(body=t.render(c))

from SimpleXMLRPCServer import SimpleXMLRPCDispatcher
dispatcher = SimpleXMLRPCDispatcher(allow_none=False, encoding=None)
API_KEYS = {'54ebf56de7684dba0d69bffc9702e1b4': 'ara',
        '49e08be8a8b7567f949237e5e0458400': 'arara',
        '2e26a1c3756f59f27e22dce6e3cd491f': 'portal'}
USER_KEYS = {'00000000000000000000000000000000': 'test'}
def _index(api_key, source, uri, title, content, boost=1.0, filetype='html', date=None):
    if api_key not in API_KEYS:
        return "INVALID API_KEY"
    if API_KEYS[api_key] != source:
        return "INVALID SOURCE"
    indexer_service = get_object('indexer_service', object_namespace)
    #indexer_service._setOneway(['index'])
    indexer_service.index(source, uri, title, content, boost, filetype, date=date)
    return "OK"

def _search(user_key, source, query, from_=0, to_=10, sort="score"):
    if user_key not in USER_KEYS:
        return "INVALID USER KEY"
    search_service = get_object('searcher_service', object_namespace)
    result = search_service.search(query, from_=from_, to_=to_, source=source, sort=sort)
    return result

dispatcher.register_function(_index, 'index')
dispatcher.register_function(_search, 'search')

def api(request):
    if request['REQUEST_METHOD'] == 'POST':
        raw_post_data = request['wsgi.input'].read(int(request.get('CONTENT_LENGTH')))
        result = dispatcher._marshaled_dispatch(raw_post_data)
        return Response(body=result,
            headers={'content-type': 'text/xml'})
    raise Exception('Use POST please')

def _was_modified_since(header_date=None, mtime=0, size=0):
    """HTTP 헤더의 정보와 실제 파일의 정보를 이용하여 새로 변경된 파일인지 판단하는 함수.
    (원본 코드는 http://code.djangoproject.com/browser/django/trunk/django/views/static.py 참조)"""

    try:
        if header_date is None:
            raise ValueError
        matches = re.match(r"^([^;]+)(; length=([0-9]+))?$", header_date, re.IGNORECASE)
        header_mtime = mktime_tz(parsedate_tz(matches.group(1)))
        header_len = matches.group(3)
        if header_len and int(header_len) != size:
            raise ValueError
        if mtime > header_mtime:
            raise ValueError
    except (AttributeError, ValueError):
        return True
    return False

def media(request):
    """CSS, Javascript, 이미지 등 일반 데이터 파일을 그대로 보내주는 핸들러"""

    path = os.path.join(os.path.dirname(__file__), request['PATH_INFO'].lstrip('/'))
    if (not os.path.exists(path)):
        return ResponseNotFound(url=request['PATH_INFO'])
    mimetype = mimetypes.guess_type(path)[0] or 'application/octet-stream'
    statobj = os.stat(path)
    http_modified_since = request.has_key('HTTP_IF_MODIFIED_SINCE') and request['HTTP_IF_MODIFIED_SINCE'] or None
    if not _was_modified_since(http_modified_since, statobj[fstat.ST_MTIME], statobj[fstat.ST_SIZE]):
        return ResponseNotModified(http_date(statobj[fstat.ST_MTIME]))
    try:
        data = open(path, 'rb').read()
    except IOError:
        return ResponseError(msg='Cannot read the file "%s"' % path)
    return Response(body=data, headers={
        'content-type': mimetype,
        'content-length': len(data),
        'last-modified': http_date(statobj[fstat.ST_MTIME])
    })

def feedback(request):
    t = loader.get_template('feedback.html')
    c = Context({
        'page_name': 'feedback',
    })
    return Response(body=t.render(c))

def about(request):
    t = loader.get_template('about.html')
    c = Context({
        'page_name': 'about',
    })
    return Response(body=t.render(c))
