#!/usr/bin/env python

from __future__ import absolute_import

import thread_hack
import Pyro.core
import Pyro.naming
import Pyro.util

import optparse
import sys
import os

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from ksearch.search.documentindexer import DocumentIndexer 

if __name__ == '__main__':
    Pyro.core.initClient(banner=0)

    logger = Pyro.util.UserLogger()

    source_name = "DocumentIndexer"

    logger.msg(source_name, "Welcome to URL Queue Server, initializing...")
    parser = optparse.OptionParser()
    parser.add_option("-n", "--namespace", dest="object_namespace", default="ksearch_main",
                      help="Object name to register in NS")
    options, args = parser.parse_args()
    logger.msg(source_name, "Args parsed:", repr(options.__dict__))

    logger.msg(source_name, "Finding nameserver...")
    Pyro.core.initServer()
    ns=Pyro.naming.NameServerLocator().getNS()

    logger.msg(source_name, "Getting Index Queue...")
    idx_queue = Pyro.core.getProxyForURI("PYRONAME://:%s.idxqueue"%options.object_namespace)

    logger.msg(source_name, "Getting IndexerService...")
    indexer_service = Pyro.core.getProxyForURI("PYRONAME://:%s.indexer_service"%options.object_namespace)
    indexer_service._setOneway(['index'])

    logger.msg(source_name, "Getting Storage...")
    storage = Pyro.core.getProxyForURI("PYRONAME://:%s.storage"%options.object_namespace)

    logger.msg(source_name, "Creating DocumentIndexer...")
    doc_indexer = DocumentIndexer(indexer_service, storage, idx_queue)
    logger.msg(source_name, "Starting DocumentIndexer...")

    def clean_up(signal, frame):
        logger.msg(source_name, "Cleaning up document indexer...")
        sys.exit(0)

    import signal
    signal.signal(signal.SIGTERM, clean_up)
    signal.signal(signal.SIGQUIT, clean_up)

    doc_indexer.start()
