#!/bin/bash

export PYTHONPATH="$PYTHONPATH:.."

if [ "$1" == "" ]; then
	NAMESPACE="test_world"
else
	NAMESPACE="$1"
fi

if [ -z $BACKEND ]; then BACKEND=lucene; fi
if [ -z $DOWNLOAD ]; then DOWNLOAD=1; fi

PIDS=""

echo "Starting pyro NS..."
pyro-ns >>  output.log 2>>  output.log &
PIDS="$PIDS $!"

sleep 2

echo "Starting storage server..."
./run_storage.py --namespace="$NAMESPACE" >>  output.log 2>>  output.log &
PIDS="$PIDS $!"
sleep 2

echo "Starting database server..."
./run_database.py --namespace="$NAMESPACE" >>  output.log 2>>  output.log &
PIDS="$PIDS $!"
sleep 2

echo "Starting crawler server..."
./run_crawler_server.py --namespace="$NAMESPACE" >>  output.log 2>>  output.log &
PIDS="$PIDS $!"
sleep 2

echo "Starting indexer service..."
./run_indexer_service.py --namespace="$NAMESPACE" --backend=$BACKEND >> output.log 2>> output.log &
PIDS="$PIDS $!"
sleep 4

echo "Starting searcher service..."
./run_searcher_service.py --namespace="$NAMESPACE" --backend=$BACKEND >>  output.log 2>>  output.log &
PIDS="$PIDS $!"
sleep 2

echo "Starting document indexer..."
./run_document_indexer.py --namespace="$NAMESPACE" >>  output.log 2>>  output.log &
PIDS="$PIDS $!"

echo "Starting $DOWNLOAD downloaders..."
for i in `seq $DOWNLOAD`; do
    ./run_downloader.py --namespace="$NAMESPACE" >>output.log 2>>output.log &
    PIDS="$PIDS $!"
done

if [ "$2" == "" ]; then
	echo "Starting Sparcs Web Server..."
	python test_server.py --namespace="$NAMESPACE" >>  output.log 2>>  output.log &
	PIDS="$PIDS $!"
else
	echo "Starting Sparcs Web Server on port $2..."
	python test_server.py --namespace="$NAMESPACE" --port=$2 >>  output.log 2>>  output.log &
	PIDS="$PIDS $!"
fi

if [ "$3" == "" ]; then
	echo "Starting adding default seed urls..."
        python add_url.py --namespace="$NAMESPACE" < seed_urls >> output.log 2>> output.log &
else
	echo "Starting adding seed url $3..."
	python add_url.py --namespace="$NAMESPACE" < $3 >> output.log 2>>  output.log &
fi

echo "$PIDS" > system.pids
#tail -f output.log
