# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil

from ksearch.search import indexerservice
from ksearch.crawler import database

class IndexerServiceTestCase(unittest.TestCase):
    def setUp(self):
        self.index_dir = tempfile.mkdtemp()
        self.tmp_dir = tempfile.mkdtemp()
        self.database = database.Database(self.tmp_dir)
        self.indexsrv = indexerservice.IndexerService(self.index_dir, lambda: self.database)

    def tearDown(self):
	del self.database
        del self.indexsrv
	shutil.rmtree(self.tmp_dir)
        shutil.rmtree(self.index_dir)

    def test_string(self):
        self.indexsrv.index('web', 'http://example.com', 'test title', 'test content')
        self.indexsrv.index('web', 'http://example.net', 'test title', 'test content')
        self.assertEquals(self.indexsrv.doc_count(), 2)

    def test_unicode(self):
        self.indexsrv.index('web', u'http://example.com', u'test title', u'test content')
        self.indexsrv.index('web', u'http://example.net', u'test title', u'test content')
        self.assertEquals(self.indexsrv.doc_count(), 2)

    def test_replace(self):
        self.indexsrv.index('web', 'http://example.com', 'test title', 'test content')
        self.indexsrv.flush()
        self.assertEquals(self.indexsrv.doc_count(), 1)
        self.indexsrv.index('web', 'http://example.com', 'test title', 'test content')
        self.assertEquals(self.indexsrv.doc_count(), 2)
        self.indexsrv.optimize()
        self.assertEquals(self.indexsrv.doc_count(), 1)


def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(IndexerServiceTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
