# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil

from ksearch.search import indexerservice
from ksearch.search import searcherservice

from ksearch.crawler import database

class SearcherServiceTestCase(unittest.TestCase):
    def setUp(self):
        self.index_dir = tempfile.mkdtemp()
        self.tmp_dir = tempfile.mkdtemp()
        self.database = database.Database(self.tmp_dir)
        self.indexsrv = indexerservice.IndexerService(self.index_dir, lambda: self.database)
        self.searchsrv = searcherservice.SearcherService(self.index_dir)

    def tearDown(self):
	del self.database
        del self.indexsrv
	shutil.rmtree(self.tmp_dir)
        shutil.rmtree(self.index_dir)

    def test_search(self):
        self.indexsrv.index('web', 'http://example.com', 'test title', 'test content')
        self.indexsrv.index('web', 'http://example2.com', 'test title', 'test content apple')
        self.indexsrv.index('web', 'http://example3.com', 'test title', 'test content pie')
        self.indexsrv.index('web', 'http://example4.com', 'test title', u'한글을 사랑합시다')
        self.indexsrv.index('ara', 'http://ara.kaist.ac.kr', 'asdf', u'qwert')

        # index is not applied to the files
        self.searchsrv.reload()
        hits = self.searchsrv.search(u'사랑합시다')
        self.assertEquals(hits['results'], 0)

        # index is now applied
        self.indexsrv.flush()
        self.assertEquals(self.indexsrv.doc_count(), 5)

        # but no use if searcher is not reloaded
        hits = self.searchsrv.search(u'test')
        self.assertEquals(hits['results'], 0)
        hits = self.searchsrv.search(u'사랑합시다')
        self.assertEquals(hits['results'], 0)

        self.searchsrv.reload()

        # now it works!
        hits = self.searchsrv.search(u'test')
        self.assertEquals(hits['results'], 4)
        hits = self.searchsrv.search(u'사랑합시다')
        self.assertEquals(hits['results'], 1)
        hits = self.searchsrv.search(u'qwert')
        self.assertEquals(hits['results'], 0)
        hits = self.searchsrv.search(u'qwert', source='ara')
        self.assertEquals(hits['results'], 1)

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(SearcherServiceTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
