# coding: utf-8

import logging
import re
import time

import xapian

pattern = re.compile(u'[0-9A-Za-z가-힣]+')

SOURCE, URI, TITLE = range(3)

class IndexerService(object):

    def __init__(self, index_dir, log_db):
        self.index_dir = index_dir
        self.log_db = log_db

        self.database = xapian.WritableDatabase(
            index_dir, xapian.DB_CREATE_OR_OPEN)

    def index(self, source, uri, title, content):
        document = xapian.Document()
        document.add_value(SOURCE, source)
        document.add_value(URI, uri.encode('utf-8'))
        document.add_value(TITLE, title.encode('utf-8'))
        document.set_data(content)

        for pos, match in enumerate(pattern.finditer(content)):
            word = match.group(0)
            document.add_posting(word, pos)

        self.database.add_document(document)
        self.database.flush()
        self.log_db.increment('statistics', 'total_indexed_page')

class SearcherService(object):

    def __init__(self, index_dir):
        self.index_dir = index_dir

        self.parser = xapian.QueryParser()
        self.logger = logging.getLogger('search')

    def reload(self):
        database = xapian.Database(self.index_dir)
        self.searcher = xapian.Enquire(database)

    def search(self, q, from_=0, to_=10):
        start = time.time()
        query = self.parser.parse_query(q)
        self.searcher.set_query(query)
        matches = self.searcher.get_mset(from_, to_)
        results = matches.get_matches_estimated()
        end = time.time()
        duration = end - start

        self.logger.info("search of '%s' took %f seconds and got %d results"
            % (q, duration, results))

        hits = []
        for match in matches:
            item = {}
            document = match.document
            item['content'] = u''
            item['title'] = document.get_value(TITLE).decode('utf-8')
            item['uri'] = document.get_value(URI).decode('utf-8')
            hits.append(item)

        result = {'results': results, 'hits': hits}
        return result
